package java.util.stream;

import java.util.Iterator;
import java.util.function.DoubleSupplier;
import java.util.Objects;
import java.util.Arrays;
import java.util.Spliterators;
import java.util.Spliterator;
import java.util.PrimitiveIterator;
import java.util.DoubleSummaryStatistics;
import java.util.function.BiConsumer;
import java.util.function.ObjDoubleConsumer;
import java.util.function.Supplier;
import java.util.OptionalDouble;
import java.util.function.DoubleBinaryOperator;
import java.util.function.DoubleConsumer;
import java.util.function.DoubleToLongFunction;
import java.util.function.DoubleToIntFunction;
import java.util.function.DoubleFunction;
import java.util.function.DoubleUnaryOperator;
import java.util.function.DoublePredicate;

public interface DoubleStream extends BaseStream<Double, DoubleStream>
{
    public abstract DoubleStream filter(DoublePredicate p0);
    
    public abstract DoubleStream map(DoubleUnaryOperator p0);
    
    public abstract <U> Stream<U> mapToObj(DoubleFunction<? extends U> p0);
    
    public abstract IntStream mapToInt(DoubleToIntFunction p0);
    
    public abstract LongStream mapToLong(DoubleToLongFunction p0);
    
    public abstract DoubleStream flatMap(DoubleFunction<? extends DoubleStream> p0);
    
    public abstract DoubleStream distinct();
    
    public abstract DoubleStream sorted();
    
    public abstract DoubleStream peek(DoubleConsumer p0);
    
    public abstract DoubleStream limit(long p0);
    
    public abstract DoubleStream skip(long p0);
    
    public abstract void forEach(DoubleConsumer p0);
    
    public abstract void forEachOrdered(DoubleConsumer p0);
    
    public abstract double[] toArray();
    
    public abstract double reduce(double p0, DoubleBinaryOperator p1);
    
    public abstract OptionalDouble reduce(DoubleBinaryOperator p0);
    
    public abstract <R> R collect(Supplier<R> p0, ObjDoubleConsumer<R> p1, BiConsumer<R, R> p2);
    
    public abstract double sum();
    
    public abstract OptionalDouble min();
    
    public abstract OptionalDouble max();
    
    public abstract long count();
    
    public abstract OptionalDouble average();
    
    public abstract DoubleSummaryStatistics summaryStatistics();
    
    public abstract boolean anyMatch(DoublePredicate p0);
    
    public abstract boolean allMatch(DoublePredicate p0);
    
    public abstract boolean noneMatch(DoublePredicate p0);
    
    public abstract OptionalDouble findFirst();
    
    public abstract OptionalDouble findAny();
    
    public abstract Stream<Double> boxed();
    
    public abstract DoubleStream sequential();
    
    public abstract DoubleStream parallel();
    
    public abstract PrimitiveIterator.OfDouble iterator();
    
    public abstract Spliterator.OfDouble spliterator();
    
    public static DoubleStream.Builder builder() {
        return new Streams.DoubleStreamBuilderImpl();
    }
    
    public static DoubleStream empty() {
        return StreamSupport.doubleStream(Spliterators.emptyDoubleSpliterator(), false);
    }
    
    public static DoubleStream of(double n) {
        return StreamSupport.doubleStream(new Streams.DoubleStreamBuilderImpl(n), false);
    }
    
    public static DoubleStream of(double... array) {
        return Arrays.stream(array);
    }
    
    public static DoubleStream iterate(final double n, final DoubleUnaryOperator obj) {
        Objects.requireNonNull(obj);
        return StreamSupport.doubleStream(Spliterators.spliteratorUnknownSize(new PrimitiveIterator.OfDouble() {
            double t = n;
            
            @Override
            public boolean hasNext() {
                return true;
            }
            
            @Override
            public double nextDouble() {
                double t = this.t;
                this.t = obj.applyAsDouble(this.t);
                return t;
            }
        }, 1296), false);
    }
    
    public static DoubleStream generate(DoubleSupplier obj) {
        Objects.requireNonNull(obj);
        return StreamSupport.doubleStream(new StreamSpliterators.InfiniteSupplyingSpliterator.OfDouble(Long.MAX_VALUE, obj), false);
    }
    
    public static DoubleStream concat(DoubleStream obj, DoubleStream obj2) {
        Objects.requireNonNull(obj);
        Objects.requireNonNull(obj2);
        return (DoubleStream)StreamSupport.doubleStream(new Streams.ConcatSpliterator.OfDouble(obj.spliterator(), obj2.spliterator()), obj.isParallel() || obj2.isParallel()).onClose(Streams.composedClose(obj, obj2));
    }
    
    public interface Builder extends DoubleConsumer
    {
        public abstract void accept(double p0);
        
        default public DoubleStream.Builder add(double n) {
            this.accept(n);
            return this;
        }
        
        public abstract DoubleStream build();
    }
}
